<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Teacher_Discipline_Edit extends Controller_Environment_Teacher
{
    /** @var  Model_Discipline */
    private $discipline;

    const twigSource = 'teacher/discipline/edit/';


    public function before() {
        parent::before();

        $id = (int) $this->request->param('id');
        $this->discipline = Model_Discipline::load($id);
        if ($this->discipline->AuthorID != $this->user->TeacherID)
            throw HTTP_Exception::factory(403, "Недостаточно прав для редактирования.");

        $section = $this->request->param('section');
        $this->twig->set_filename(static::twigSource . $section);
        $this->twig->Discipline = $this->discipline;
    }


    public function action_edit_settings() {
        $this->twig->SubjectsList = Model_Faculty::with($this->user->FacultyID)->getSubjects();
        $this->twig->GradesList   = Model_Grades::toArray();
    }

    public function action_edit_structure() {
        $map = Model_Map::getMapForDiscipline($this->discipline->ID);
        $this->twig->Map = $this->getMapInfo($map);
    }
    
    public function action_edit_groups() {
        $this->twig->set([
            'GroupsForDiscipline' => $this->discipline->getGroups(),
            'Groups'              => Model_Faculty::with($this->discipline->FacultyID)
                                        ->getGroups($this->discipline->GradeID),
        ]);
    }
    
    public function action_edit_students() {
        $this->manageGroups($this->discipline, $students, $attached);

        $this->twig->set([
            'Groups'         => $students,
            'GroupsAttached' => $attached,
            'GradesList'     => Model_Grades::toArray(),
            'GroupsList'     => Model_Faculty::with($this->discipline->FacultyID)
                                    ->getGroups($this->discipline->GradeID),
        ]);
    }
    
    public function action_edit_teachers() {
        $this->twig->set([
            'FacultiesList'    => Model_Faculties::load(),
            'Departments'      => Model_Departments::byFaculty($this->user->FacultyID),
            # TODO: getPersonalInfo deprecated
            # TODO: getPersonalInfo check param (use userId instead teacherId)
            //'Author'           => Model_Account::getPersonalInfo($this->discipline->AuthorID),
            'BindTeachersList' => Model_Teachers::getNamesForDiscipline($this->discipline->ID),
        ]);
    }


    private function getMapInfo($map) {
        $maxRate = $i = $module = 0;
        $mapHandled = [];

        foreach ($map as $row) {
            if ($row['ModuleType'] == 'extra') {
                continue;
            }

            if ($row['ModuleID'] != $module) {
                $module = $row['ModuleID'];
                $i++;

                $mapHandled[$i]['ModuleID'] = $row['ModuleID'];
                $mapHandled[$i]['ModuleType'] = $row['ModuleType'];

                if ($row['ModuleType'] == 'bonus') {
                    $mapHandled['isSetBonus'] = true;
                }
            }

            $mapHandled[$i]['IsExam'] = $row['IsExam'];
            $mapHandled[$i]['MaxRate'] += (int) $row['MaxRate'];
            $mapHandled[$i]['ModuleTitle'] = $row['ModuleName'];
            $mapHandled[$i]['SubmodulesCount'] += 1;

            $mapHandled[$i][] = [
                'SubmoduleID'      => $row['SubmoduleID'],
                'Title'            => $row['SubmoduleName'],
                'Description'      => $row['SubmoduleDescription'],
                'SubmoduleControl' => $row['SubmoduleType'],
                'MaxRate'          => (int) $row['MaxRate'],
            ];

            $maxRate += $row['MaxRate'];

            $type = $row['SubmoduleType'];
            $mapHandled[$i][$type] += (int) $row['MaxRate'];
        }

        $mapHandled['ModulesCount'] = $i;
        $mapHandled['MaxRate'] = (int) $maxRate;

        if ($mapHandled['isSetBonus'] == true) {
            $mapHandled['MaxRate'] = $mapHandled['MaxRate'] - 10;
        }

        return $mapHandled;
    }

    private function manageGroups(Model_Discipline $discipline, &$std, &$attached) {
        $students = Model_Students::ofDiscipline($discipline->ID);
        $groupID = $i = $k = $n = $j = 0;
        $attached = $std = [];

        foreach ($students as $row) {
            if ($row['GroupID'] != $groupID) {
                $groupID = $row['GroupID'];

                if ($row['AttachType'] == 'attach') {
                    $attached[++$k] = $row;
                } else {
                    $std[++$i] = $row;
                }
            }

            if ($row['AttachType'] == 'attach') {
                $attached[$k]['students'][] = $row;
            } else {
                $std[$i]['students'][] = $row;
            }
        }
    }
}
            